@@ -1,3 +1,4 @@
+"""ViSP solar calibration task."""
 import logging
 from typing import List
 from typing import Tuple
@@ -13,35 +14,88 @@ from dkist_processing_math.arithmetic import subtract_array_from_arrays
 from dkist_processing_math.statistics import average_numpy_arrays
 
 from dkist_processing_visp.models.tags import VispTag
-from dkist_processing_visp.visp_base import VispScienceTask
+from dkist_processing_visp.tasks.mixin.corrections import CorrectionsMixin
+from dkist_processing_visp.tasks.mixin.input_frame_loaders import InputFrameLoadersMixin
+from dkist_processing_visp.tasks.mixin.intermediate_frame_helpers import (
+    IntermediateFrameHelpersMixin,
+)
+from dkist_processing_visp.tasks.visp_base import VispTaskBase
 
 
-class SolarCalibration(VispScienceTask, QualityMixin):
+class SolarCalibration(
+    VispTaskBase,
+    IntermediateFrameHelpersMixin,
+    InputFrameLoadersMixin,
+    CorrectionsMixin,
+    QualityMixin,
+):
     """
-    Task for generating Solar Gain images for each beam/modstate
+    Task class for generating Solar Gain images for each beam/modstate.
+
+    Parameters
+    ----------
+    recipe_run_id : int
+        id of the recipe run used to identify the workflow run this task is part of
+    workflow_name : str
+        name of the workflow to which this instance of the task belongs
+    workflow_version : str
+        version of the workflow to which this instance of the task belongs
+
     """
 
+    record_provenance = True
+
     def run(self) -> None:
-        with self.apm_step("Generating solar gains"):
-            for beam in range(1, self.num_beams + 1):
-                for modstate in range(1, self.num_modulator_states + 1):
-                    logging.info(f"Dark/lamp/geometric corrections for {beam=} and {modstate=}")
+        """
+        For each beam.
+
+            For each modstate:
+                - Do dark, lamp, and geometric corrections
+                - Compute the characteristic spectra
+                - Re-apply the spectral curvature to the characteristic spectra
+                - Re-apply angle and state offset distortions to the characteristic spectra
+                - Remove the distorted characteristic solar spectra from the original spectra
+                - Write master solar gain
+
+        Returns
+        -------
+        None
+
+        """
+        for beam in range(1, self.constants.num_beams + 1):
+            for modstate in range(1, self.constants.num_modstates + 1):
+                apm_str = f"{beam = } and {modstate = }"
+                logging.info(f"Dark/lamp/geometric corrections for {apm_str}")
+                with self.apm_processing_step(f"Initial corrections for {apm_str}"):
                     self.do_initial_corrections(beam=beam, modstate=modstate)
+
+                with self.apm_processing_step(f"Computing characteristic spectra for {apm_str}"):
                     char_spec = self.compute_characteristic_spectra(beam=beam, modstate=modstate)
+
+                with self.apm_processing_step(f"Reshifting characteristic spectra for {apm_str}"):
                     reshifted_char_spec = self.reshift_characteristic_spectra(
                         char_spec=char_spec, beam=beam, modstate=modstate
                     )
+
+                with self.apm_processing_step(
+                    f"Re-distorting characteristic spectra for {apm_str}"
+                ):
                     distorted_char_spec = self.distort_characteristic_spectra(
                         char_spec=reshifted_char_spec, beam=beam, modstate=modstate
                     )
+
+                with self.apm_processing_step(f"Removing solar signal from {apm_str}"):
                     final_gain = self.remove_solar_signal(
                         char_solar_spectra=distorted_char_spec, beam=beam, modstate=modstate
                     )
+
+                with self.apm_writing_step(f"Writing solar gain for {apm_str}"):
                     self.write_solar_gain_calibration(
                         gain_array=final_gain, beam=beam, modstate=modstate
                     )
-        with self.apm_step("Finding number of input solar gain frames."):
-            no_of_raw_solar_frames: int = self.count(
+
+        with self.apm_processing_step("Computing and logging quality metrics"):
+            no_of_raw_solar_frames: int = self.scratch.count_all(
                 tags=[
                     VispTag.input(),
                     VispTag.frame(),
@@ -49,34 +103,84 @@ class SolarCalibration(VispScienceTask, QualityMixin):
                 ],
             )
 
-        with self.apm_step("Sending solar gain frame count for quality metric storage"):
             self.quality_store_task_type_counts(
                 task_type="SOLAR_GAIN", total_frames=no_of_raw_solar_frames
             )
 
     def dark_only_modstate_data(self, beam: int, modstate: int) -> np.ndarray:
-        """ Array for a single beam/modstate that has ONLY had the dark signal removed """
-        array_generator = self.load_intermediate_arrays(
+        """
+        Array for a single beam/modstate that has ONLY had the dark signal removed.
+
+        Parameters
+        ----------
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Array with dark signal removed
+
+        """
+        array_generator = self.intermediate_frame_helpers_load_intermediate_arrays(
             beam=beam, task="SC_DARK_ONLY", modstate=modstate
         )
         return next(array_generator)
 
     def unshifted_geo_corrected_modstate_data(self, beam: int, modstate: int) -> np.ndarray:
-        """ Array for a single beam/modstate that has dark, lamp, angle, and state offset corrections """
-        array_generator = self.load_intermediate_arrays(
+        """
+        Array for a single beam/modstate that has dark, lamp, angle, and state offset corrections.
+
+        Parameters
+        ----------
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Array with dark signal, lamp signal, angle and state offset removed
+
+        """
+        array_generator = self.intermediate_frame_helpers_load_intermediate_arrays(
             beam=beam, task="SC_GEO_NOSHIFT", modstate=modstate
         )
         return next(array_generator)
 
     def geo_corrected_modstate_data(self, beam: int, modstate: int) -> np.ndarray:
-        """ Array for a single beam/modstate that has dark, lamp, and ALL of the geometric corrects """
-        array_generator = self.load_intermediate_arrays(
+        """
+        Array for a single beam/modstate that has dark, lamp, and ALL of the geometric corrects.
+
+        Parameters
+        ----------
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Array with dark signal, and lamp signal removed, and all geometric corrections made
+        """
+        array_generator = self.intermediate_frame_helpers_load_intermediate_arrays(
             beam=beam, task="SC_GEO_ALL", modstate=modstate
         )
         return next(array_generator)
 
     def do_initial_corrections(self, beam: int, modstate: int) -> None:
-        """Do dark, lamp, and geometric corrections for all data that will be used
+        """
+        Do dark, lamp, and geometric corrections for all data that will be used.
 
         At two intermediate points the current arrays are saved because they'll be needed by various helpers:
 
@@ -87,19 +191,36 @@ class SolarCalibration(VispScienceTask, QualityMixin):
         SC_GEO_NOSHIFT - The solar gain arrays after dark, lamp, angle, and state offset correction. In other words,
                          they do not have spectral curvature removed. These are used to reshift the characteristic
                          spectra to the original spectral curvature.
+
+        Parameters
+        ----------
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        None
         """
-        for exp_time in self.solar_exposure_times:
+        for exp_time in self.constants.solar_exposure_times:
             try:
-                dark_array = self.load_intermediate_dark_array(beam=beam, exposure_time=exp_time)
+                dark_array = self.intermediate_frame_helpers_load_dark_array(
+                    beam=beam, exposure_time=exp_time
+                )
             except StopIteration:
                 raise ValueError(f"No matching dark found for {exp_time = } s")
 
             logging.info(f"Doing dark, lamp, and geo corrections for {beam=} and {modstate=}")
             ## Load frames
-            input_solar_arrays = self.input_solar_gain_array_generator(
+            input_solar_arrays = self.input_frame_loaders_solar_gain_array_generator(
                 beam=beam, modstate=modstate, exposure_time=exp_time
             )
-            lamp_array = self.load_intermediate_lamp_gain_array(beam=beam, modstate=modstate)
+            lamp_array = self.intermediate_frame_helpers_load_lamp_gain_array(
+                beam=beam, modstate=modstate
+            )
 
             ## Average
             avg_solar_array = average_numpy_arrays(input_solar_arrays)
@@ -109,7 +230,7 @@ class SolarCalibration(VispScienceTask, QualityMixin):
                 subtract_array_from_arrays(arrays=avg_solar_array, array_to_subtract=dark_array)
             )
             # Save the only-dark-corr because this will be used to make the final Solar Gain object
-            self.write_intermediate_arrays(
+            self.intermediate_frame_helpers_write_arrays(
                 arrays=dark_corrected_solar_array, beam=beam, modstate=modstate, task="SC_DARK_ONLY"
             )
 
@@ -121,33 +242,48 @@ class SolarCalibration(VispScienceTask, QualityMixin):
             )
 
             ## Geo correction
-            angle = self.get_angle(beam=beam)
-            state_offset = self.get_state_offset(beam=beam, modstate=modstate)
-            spec_shift = self.get_spec_shift(beam=beam)
+            angle = self.intermediate_frame_helpers_load_angle(beam=beam)
+            state_offset = self.intermediate_frame_helpers_load_state_offset(
+                beam=beam, modstate=modstate
+            )
+            spec_shift = self.intermediate_frame_helpers_load_spec_shift(beam=beam)
 
             geo_corrected_array = next(
-                self.correct_geometry(lamp_corrected_solar_array, state_offset, angle)
+                self.corrections_correct_geometry(lamp_corrected_solar_array, state_offset, angle)
             )
             # We need unshifted, but geo-corrected arrays for reshifting and normalization
-            self.write_intermediate_arrays(
+            self.intermediate_frame_helpers_write_arrays(
                 arrays=geo_corrected_array, beam=beam, modstate=modstate, task="SC_GEO_NOSHIFT"
             )
 
             # Now finish the spectral shift correction
             spectral_corrected_array = next(
-                self.remove_spec_geometry(geo_corrected_array, spec_shift)
+                self.corrections_remove_spec_geometry(geo_corrected_array, spec_shift)
             )
-            self.write_intermediate_arrays(
+            self.intermediate_frame_helpers_write_arrays(
                 arrays=spectral_corrected_array, beam=beam, modstate=modstate, task="SC_GEO_ALL"
             )
 
     def compute_characteristic_spectra(self, beam: int, modstate: int) -> np.ndarray:
-        """Compute the characteristic spectra via a moving average along the slit
-
-        Also, identify and ignore hairlines
         """
-        spectral_avg_window = self.input_dataset_parameters_get("visp_solar_spectral_avg_window")
-        hairline_fraction = self.input_dataset_parameters_get("visp_solar_hairline_fraction")
+        Compute the characteristic spectra via a moving average along the slit, identify and ignore hairlines.
+
+        Parameters
+        ----------
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Characteristic spectra array
+        """
+        spectral_avg_window = self.parameters.solar_spectral_avg_window
+        hairline_fraction = self.parameters.solar_hairline_fraction
 
         logging.info(
             f"Computing characteristic spectra for {beam=} and {modstate=} using {spectral_avg_window=} and {hairline_fraction=}"
@@ -170,15 +306,32 @@ class SolarCalibration(VispScienceTask, QualityMixin):
     def reshift_characteristic_spectra(
         self, char_spec: np.ndarray, beam: int, modstate: int
     ) -> np.ndarray:
-        """Re-apply the spectral curvature to the characteristic spectra
+        """
+        Re-apply the spectral curvature to the characteristic spectra.
 
-        This actually re-fits the shifts instead of using the pre-computed shifts, which seems a little weird, but is
+        Note: This function re-fits the shifts instead of using the pre-computed shifts, which seems a little weird, but is
         empirically required. Importantly, the fit metric for this re-shift is different than that used to compute the
         initial shift. Here we actually minimize residuals in the final gain image.
-        """
 
+        Parameters
+        ----------
+        char_spec : np.ndarray
+            Computed characteristic spectra
+
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Re-shifted characteristic spectra array
+        """
         # Grab initial guesses
-        spec_shifts = self.get_spec_shift(beam=beam)
+        spec_shifts = self.intermediate_frame_helpers_load_spec_shift(beam=beam)
         num_spec = spec_shifts.size
 
         # Grab unshifted spectra that will be the shift target
@@ -188,13 +341,11 @@ class SolarCalibration(VispScienceTask, QualityMixin):
 
         logging.info(f"Computing line zones for {beam=} and {modstate=}")
         zone_kwargs = {
-            "prominence": self.input_dataset_parameters_get("visp_solar_zone_prominence"),
-            "width": self.input_dataset_parameters_get("visp_solar_zone_width"),
-            "bg_order": self.input_dataset_parameters_get("visp_solar_zone_bg_order"),
-            "normalization_percentile": self.input_dataset_parameters_get(
-                "visp_solar_zone_normalization_percentile"
-            ),
-            "rel_height": self.input_dataset_parameters_get("visp_solar_zone_rel_height"),
+            "prominence": self.parameters.solar_zone_prominence,
+            "width": self.parameters.solar_zone_width,
+            "bg_order": self.parameters.solar_zone_bg_order,
+            "normalization_percentile": self.parameters.solar_zone_normalization_percentile,
+            "rel_height": self.parameters.solar_zone_rel_height,
         }
         zones = self.compute_line_zones(char_spec, **zone_kwargs)
         logging.info(f"Found {zones=} for {beam=} and {modstate=}")
@@ -219,19 +370,62 @@ class SolarCalibration(VispScienceTask, QualityMixin):
     def distort_characteristic_spectra(
         self, char_spec: np.ndarray, beam: int, modstate: int
     ) -> np.ndarray:
-        """ Re-apply angle and state offset distortions to the characteristic spectra """
-        logging.info(f"Re-distorting characteristic spectra for {beam=} and {modstate=}")
-        angle = self.get_angle(beam=beam)
-        state_offset = self.get_state_offset(beam=beam, modstate=modstate)
+        """
+        Re-apply angle and state offset distortions to the characteristic spectra.
 
-        distorted_spec = next(self.correct_geometry(char_spec, -1 * state_offset, -1 * angle))
+        Parameters
+        ----------
+        char_spec : np.ndarray
+            Computed characteristic spectra
+
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Characteristic spectra array with angle and offset distortions re-applied
+        """
+        logging.info(f"Re-distorting characteristic spectra for {beam=} and {modstate=}")
+        angle = self.intermediate_frame_helpers_load_angle(beam=beam)
+        state_offset = self.intermediate_frame_helpers_load_state_offset(
+            beam=beam, modstate=modstate
+        )
+
+        distorted_spec = next(
+            self.corrections_correct_geometry(char_spec, -1 * state_offset, -1 * angle)
+        )
 
         return distorted_spec
 
     def remove_solar_signal(
         self, char_solar_spectra: np.ndarray, beam: int, modstate: int
     ) -> np.ndarray:
-        """ Remove the distorted characteristic solar spectra from the OG spectra """
+        """
+        Remove the distorted characteristic solar spectra from the original spectra.
+
+        Parameters
+        ----------
+        char_solar_spectra : np.ndarray
+            Characteristic solar spectra
+
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        np.ndarray
+            Original spectral array with characteristic solar spectra removed
+
+        """
         logging.info(f"Removing characteristic solar spectra from {beam=} and {modstate=}")
         raw_gain = self.dark_only_modstate_data(beam=beam, modstate=modstate)
 
@@ -242,9 +436,27 @@ class SolarCalibration(VispScienceTask, QualityMixin):
     def write_solar_gain_calibration(
         self, gain_array: np.ndarray, beam: int, modstate: int
     ) -> None:
-        """ Write a solar gain array for a single beam and modstate """
+        """
+        Write a solar gain array for a single beam and modstate.
+
+        Parameters
+        ----------
+        gain_array: np.ndarray
+            Solar gain array
+
+        beam : int
+            The beam number for this array
+
+        modstate : int
+            The modulator state for this array
+
+
+        Returns
+        -------
+        None
+        """
         logging.info(f"Writing final SolarGain for {beam=} and {modstate=}")
-        self.write_intermediate_arrays(
+        self.intermediate_frame_helpers_write_arrays(
             arrays=gain_array, beam=beam, modstate=modstate, task="SOLAR_GAIN"
         )
 
@@ -270,7 +482,34 @@ class SolarCalibration(VispScienceTask, QualityMixin):
         normalization_percentile: int = 99,
         rel_height=0.97,
     ) -> List[Tuple[int, int]]:
-        """ Identify spectral regions around strong spectra features """
+        """
+        Identify spectral regions around strong spectra features.
+
+        Parameters
+        ----------
+        spec_2d : np.ndarray
+            Data
+
+        prominence : float
+            Zone prominence threshold used to identify strong spectral features
+
+        width : float
+            Zone width
+
+        bg_order : int
+            Order of polynomial fit used to remove continuum when identifying strong spectral features
+
+        normalization_percentile : int
+            Compute this percentile of the data along a specified axis
+
+
+
+        Returns
+        -------
+        List
+            List of regions to remove
+
+        """
         logging.info(
             f"Finding zones using {prominence=}, {width=}, {bg_order=}, {normalization_percentile=}, and {rel_height=}"
         )
@@ -303,7 +542,22 @@ class SolarCalibration(VispScienceTask, QualityMixin):
 
     @staticmethod
     def identify_overlapping_zones(rips: np.ndarray, lips: np.ndarray) -> List[int]:
-        """ Identify line zones that overlap with other zones. Any overlap greater than 1 pixel is flagged. """
+        """
+        Identify line zones that overlap with other zones. Any overlap greater than 1 pixel is flagged.
+
+        Parameters
+        ----------
+        rips : np.ndarray
+            Right borders of the region around the peak
+
+        lips : np.ndarray
+            Left borders of the region around the peak
+
+        Returns
+        -------
+        List
+            List of ranges to be removed
+        """
         all_ranges = [np.arange(zmin, zmax) for zmin, zmax in zip(rips, lips)]
         ranges_to_remove = []
         for i in range(len(all_ranges)):
@@ -330,28 +584,29 @@ class SolarCalibration(VispScienceTask, QualityMixin):
         spec: np.ndarray, target_spec: np.ndarray, zones: List[Tuple[int, int]], x_init: float
     ) -> float:
         """
-        Compute the shift for a single spatial position back from rectified spectra to the OG (curved) position
+        Compute the shift for a single spatial position back from rectified spectra to the original (curved) position.
 
         Line zones are used to increase the SNR of the chisq and the final shift is the mean of the shifts computed
         for each zone.
 
         Parameters
         ----------
-        spec
+        spec : np.ndarray
             The 1D spectrum to shift back
 
-        target_spec
+        target_spec : np.ndarray
             The reference spectrum. This should be the un-shifted, raw spectrum at the same position as `spec`
 
-        zones
+        zones : List
             List of zone borders (in px coords)
 
-        x_init
+        x_init: float
             Initial guess for the shift. This is used to shift the zones so it needs to be pretty good, but not perfect.
 
         Returns
         -------
-        The shift value
+        float
+            The shift value
         """
         shifts = np.zeros(len(zones))
         for i, z in enumerate(zones):
@@ -374,9 +629,29 @@ class SolarCalibration(VispScienceTask, QualityMixin):
         par: List[float], ref_spec: np.ndarray, spec: np.ndarray, idx: np.ndarray
     ) -> float:
         """
-        Non-chisq based goodness of fit calculator for compute spectral shifts
+        Non-chisq based goodness of fit calculator for computing spectral shifts.
+
+        Instead of chisq, the metric approximates the final Gain image.
+
+        Parameters
+        ----------
+        par : List
+            List of parameters for minimization
+
+        ref_spec : np.ndarray
+            Reference spectra
+
+        spec : np.ndarray
+            Data
+
+        idx : np.ndarray
+            Range of wavelength pixels that will be compared in fit
+
+        Returns
+        -------
+        float
+            Goodness of fit metric
 
-        Instead of chisq, the metric approximates the final Gain image
         """
         shift = par[0]
         shifted_spec = spnd.shift(spec, shift, mode="constant", cval=np.nan)
