@@ -1,11 +1,15 @@
 import json
+import uuid
 from dataclasses import asdict
 from dataclasses import dataclass
+from dataclasses import is_dataclass
 from random import choice
 from random import randint
 from typing import Iterable
 from typing import Optional
 from typing import Tuple
+from typing import Union
+from unittest.mock import PropertyMock
 
 import numpy as np
 import pytest
@@ -19,6 +23,10 @@ from dkist_processing_common.models.graphql import InputDatasetResponse
 from dkist_processing_common.models.graphql import RecipeInstanceResponse
 from dkist_processing_common.models.graphql import RecipeRunResponse
 
+from dkist_processing_visp.models.constants import VispConstants
+from dkist_processing_visp.models.parameters import VispParameters
+from dkist_processing_visp.models.tags import VispTag
+
 
 class VispHeaders(Spec122Dataset):
     def __init__(
@@ -43,6 +51,7 @@ class VispHeaders(Spec122Dataset):
         self.add_constant_key("VISP_006", polarimeter_mode)
         self.add_constant_key("PAC__005", "0")
         self.add_constant_key("PAC__007", "10")
+        self.add_constant_key("FILE_ID", uuid.uuid4().hex)
 
     @key_function("VISP_011")
     def date(self, key: str):
@@ -64,6 +73,9 @@ class VispHeadersValidDarkFrames(VispHeaders):
         self.add_constant_key("VISP_019", 1)
         self.add_constant_key("VISP_020", 1)
         self.add_constant_key("ID___004")
+        self.add_constant_key(
+            "WAVELNTH", 0.0
+        )  # Intentionally bad to make sure it doesn't get parsed
         self.add_constant_key("CAM__004", 1.0)
 
 
@@ -149,8 +161,6 @@ class VispHeadersValidObserveFrames(VispHeaders):
         dataset_shape: Tuple[int, ...],
         array_shape: Tuple[int, ...],
         time_delta: float,
-        num_dsps_repeats: int,
-        dsps_repeat: int,
         num_raster_steps: int,
         raster_step: int,
         num_modstates: int,
@@ -158,20 +168,14 @@ class VispHeadersValidObserveFrames(VispHeaders):
         **kwargs,
     ):
         super().__init__(dataset_shape, array_shape, time_delta, **kwargs)
-        self.num_dsps_repeats = num_dsps_repeats
         self.num_raster_steps = num_raster_steps
         self.add_constant_key("DKIST004", "observe")
         self.add_constant_key("VISP_019", num_raster_steps)
         self.add_constant_key("VISP_020", raster_step)
-        self.add_constant_key("DKIST008", num_dsps_repeats)
-        self.add_constant_key("DKIST009", dsps_repeat)
         self.add_constant_key("ID___004")
         self.add_constant_key("VISP_010", num_modstates)
         self.add_constant_key("VISP_011", modstate)
-
-    @key_function("CAM__004")
-    def exposure_time(self, key: str) -> float:
-        return 0.02 if self.index % 2 == 0 else 0.03
+        self.add_constant_key("WAVELNTH", 656.28)
 
 
 def generate_fits_frame(header_generator: Iterable, shape=None) -> fits.HDUList:
@@ -199,7 +203,7 @@ def generate_full_visp_fits_frame(
 def generate_214_l0_fits_frame(
     s122_header: fits.Header, data: Optional[np.ndarray] = None
 ) -> fits.HDUList:
-    """ Convert S122 header into 214 L0 """
+    """Convert S122 header into 214 L0"""
     if data is None:
         data = np.ones((1, 10, 10))
     translated_header = translate_spec122_to_spec214_l0(s122_header)
@@ -234,16 +238,57 @@ class Visp122ObserveFrames(VispHeaders):
         array_shape: Tuple[int, ...],
         num_steps: int = 4,
         num_exp_per_step: int = 1,
-        num_dsps_repeats: int = 5,
+        num_map_scans: int = 5,
     ):
         super().__init__(
             array_shape=array_shape,
             time_delta=10,
-            dataset_shape=(num_exp_per_step * num_steps * num_dsps_repeats,) + array_shape[-2:],
+            dataset_shape=(num_exp_per_step * num_steps * num_map_scans,) + array_shape[-2:],
         )
         self.add_constant_key("DKIST004", "observe")
 
 
+@pytest.fixture()
+def init_visp_constants_db():
+    def constants_maker(recipe_run_id: int, constants_obj):
+        if is_dataclass(constants_obj):
+            constants_obj = asdict(constants_obj)
+        constants = VispConstants(recipe_run_id=recipe_run_id, task_name="test")
+        constants._update(constants_obj)
+        return
+
+    return constants_maker
+
+
+@dataclass
+class VispConstantsDb:
+    POLARIMETER_MODE: str = "observe_polarimetric"
+    NUM_MODSTATES: int = 10
+    NUM_MAP_SCANS: int = 2
+    NUM_RASTER_STEPS: int = 3
+    NUM_BEAMS: int = 2
+    NUM_CS_STEPS: int = 18
+    NUM_SPECTRAL_BINS: int = 1
+    NUM_SPATIAL_BINS: int = 1
+    INSTRUMENT: str = "VISP"
+    AVERAGE_CADENCE: float = 10.0
+    MINIMUM_CADENCE: float = 10.0
+    MAXIMUM_CADENCE: float = 10.0
+    VARIANCE_CADENCE: float = 0.0
+    WAVELENGTH: float = 588.0
+    LAMP_EXPOSURE_TIMES: Tuple[float] = (100.0,)
+    SOLAR_EXPOSURE_TIMES: Tuple[float] = (1.0,)
+    OBSERVE_EXPOSURE_TIMES: Tuple[float] = (0.01,)
+    POLCAL_EXPOSURE_TIMES: Union[Tuple[float], Tuple] = ()
+    SPECTRAL_LINE: str = "VISP Ca II H"
+    STOKES_PARAMS: Tuple[str] = (
+        "I",
+        "Q",
+        "U",
+        "V",
+    )  # A tuple because lists aren't allowed on dataclasses
+
+
 class FakeGQLClient:
     def __init__(self, *args, **kwargs):
         pass
@@ -278,7 +323,7 @@ def recipe_run_id():
 
 
 @pytest.fixture()
-def visp_dataset():
+def visp_dataset() -> fits.Header:
     """
     A header with some common by-frame keywords
     """
@@ -293,8 +338,26 @@ def visp_dataset():
     return header_list[0]
 
 
+@pytest.fixture()
+def calibrated_visp_dataset(visp_dataset) -> fits.Header:
+    """
+    Same as visp_dataset but with a DATE-END key.
+
+    Because now that's added during ScienceCal
+    """
+    visp_dataset["DATE-END"] = "2000-01-01T12:00:00"
+
+    return visp_dataset
+
+
 @dataclass
-class VispParameters:
+class WavelengthParameter:
+    values: tuple
+    wavelength: tuple = (397.0, 588.0, 630.0, 854.0)  # This must always be in order
+
+
+@dataclass
+class VispTestingParameters:
     visp_beam_border: int = 1000
     visp_geo_num_otsu: int = 5
     visp_geo_num_theta: int = 1500
@@ -305,27 +368,60 @@ class VispParameters:
     visp_geo_poly_fit_order: int = 3
     visp_solar_spectral_avg_window: int = 50
     visp_solar_hairline_fraction: float = 0.8
-    visp_solar_zone_prominence: float = 0.2
-    visp_solar_zone_width: int = 2
-    visp_solar_zone_bg_order: int = 22
-    visp_solar_zone_normalization_percentile: float = 99
+    visp_solar_zone_prominence: WavelengthParameter = WavelengthParameter(
+        values=(0.2, 0.2, 0.3, 0.2)
+    )
+    visp_solar_zone_width: WavelengthParameter = WavelengthParameter(values=(7, 2, 3, 2))
+    visp_solar_zone_bg_order: WavelengthParameter = WavelengthParameter(values=(21, 22, 11, 22))
+    visp_solar_zone_normalization_percentile: WavelengthParameter = WavelengthParameter(
+        values=(90, 99, 90, 90)
+    )
     visp_solar_zone_rel_height: float = 0.97
     visp_max_cs_step_time_sec: float = 20.0
+    visp_polcal_num_spatial_bins: int = 3
+    visp_polcal_num_spectral_bins: int = 1
+    visp_pac_fit_mode: str = "use_M12"
+    visp_pac_init_set: str = "OCCal_VIS"
+
+
+@pytest.fixture(scope="session")
+def testing_wavelength() -> float:
+    return 588.0
 
 
 @pytest.fixture(scope="session")
 def input_dataset_document_with_simple_parameters():
-    dataset_doc_dict = dict(parameters=[])
-    value_id = randint(1000, 2000)
-    for pn, pv in asdict(VispParameters()).items():
-        values = [
-            {
-                "parameterValueId": value_id,
-                "parameterValue": json.dumps(pv),
-                "parameterValueStartDate": "1946-11-20",  # Remember Duane Allman
-            }
-        ]
-        parameter = {"parameterName": pn, "parameterValues": values}
-        dataset_doc_dict["parameters"] += [parameter]
+    def get_input_dataset_document(parameters: VispTestingParameters):
+        dataset_doc_dict = dict(parameters=[])
+        value_id = randint(1000, 2000)
+        for pn, pv in asdict(parameters).items():
+            if type(pv) is WavelengthParameter:
+                pv = asdict(pv)
+            values = [
+                {
+                    "parameterValueId": value_id,
+                    "parameterValue": json.dumps(pv),
+                    "parameterValueStartDate": "1946-11-20",  # Remember Duane Allman
+                }
+            ]
+            parameter = {"parameterName": pn, "parameterValues": values}
+            dataset_doc_dict["parameters"] += [parameter]
+        return dataset_doc_dict
 
-    return dataset_doc_dict
+    return get_input_dataset_document
+
+
+@pytest.fixture(scope="session")
+def assign_input_dataset_doc_to_task(
+    input_dataset_document_with_simple_parameters, testing_wavelength
+):
+    def update_task(task, parameters):
+        doc_path = task.scratch.workflow_base_path / "dataset_doc.json"
+        with open(doc_path, "w") as f:
+            f.write(json.dumps(input_dataset_document_with_simple_parameters(parameters)))
+        task.tag(doc_path, VispTag.input_dataset())
+        task.parameters = VispParameters(
+            task.input_dataset_parameters, wavelength=testing_wavelength
+        )
+
+    return update_task
