@@ -10,14 +10,38 @@ from random import randint
 from astropy.io import fits
 from dkist_header_validator import spec122_validator
 from dkist_header_validator import spec214_validator
-from dkist_processing_common.tasks.base import ParsedL0InputTaskBase
+from dkist_processing_common.manual import ManualProcessing
+from dkist_processing_common.tasks import QualityL1Metrics
+from dkist_processing_common.tasks import WorkflowTaskBase
 from dkist_processing_common.tasks.mixin.metadata_store import MetadataStoreMixin
 from dkist_processing_common.tasks.mixin.quality import QualityMixin
-from dkist_processing_common.tasks.quality_metrics import QualityL0Metrics
-from dkist_processing_common.tasks.quality_metrics import QualityL1Metrics
 
-from dkist_processing_visp.tasks.quality_metrics import VispQualityMetrics
-from dkist_processing_visp.tests.conftest import VispParameters
+from dkist_processing_visp.models.tags import VispTag
+from dkist_processing_visp.parsers.visp_l0_fits_access import VispL0FitsAccess
+from dkist_processing_visp.tasks.assemble_movie import AssembleVispMovie
+from dkist_processing_visp.tasks.dark import DarkCalibration
+from dkist_processing_visp.tasks.geometric import GeometricCalibration
+from dkist_processing_visp.tasks.instrument_polarization import InstrumentPolarizationCalibration
+from dkist_processing_visp.tasks.lamp import LampCalibration
+from dkist_processing_visp.tasks.make_movie_frames import MakeVispMovieFrames
+from dkist_processing_visp.tasks.parse import ParseL0VispInputData
+from dkist_processing_visp.tasks.quality_metrics import VispL0QualityMetrics
+from dkist_processing_visp.tasks.quality_metrics import VispL1QualityMetrics
+from dkist_processing_visp.tasks.science import ScienceCalibration
+from dkist_processing_visp.tasks.solar import SolarCalibration
+from dkist_processing_visp.tasks.visp_base import VispTaskBase
+from dkist_processing_visp.tasks.write_l1 import VispWriteL1Frame
+from dkist_processing_visp.tests.conftest import VispTestingParameters
+from dkist_processing_visp.tests.e2e_helpers import LoadDarkCal
+from dkist_processing_visp.tests.e2e_helpers import LoadGeometricCal
+from dkist_processing_visp.tests.e2e_helpers import LoadInstPolCal
+from dkist_processing_visp.tests.e2e_helpers import LoadLampCal
+from dkist_processing_visp.tests.e2e_helpers import LoadSolarCal
+from dkist_processing_visp.tests.e2e_helpers import SaveDarkCal
+from dkist_processing_visp.tests.e2e_helpers import SaveGeometricCal
+from dkist_processing_visp.tests.e2e_helpers import SaveInstPolCal
+from dkist_processing_visp.tests.e2e_helpers import SaveLampCal
+from dkist_processing_visp.tests.e2e_helpers import SaveSolarCal
 
 INV = False
 try:
@@ -42,35 +66,8 @@ if QRM:
 
     plt.ioff()
 
-from dkist_processing_common.manual import ManualProcessing
-from dkist_processing_common.tasks import WorkflowDataTaskBase
 
-from dkist_processing_visp.models.tags import VispTag
-from dkist_processing_visp.tasks.assemble_movie import AssembleVispMovie
-from dkist_processing_visp.tasks.dark import DarkCalibration
-from dkist_processing_visp.tasks.geometric import GeometricCalibration
-from dkist_processing_visp.tasks.instrument_polarization import InstrumentPolarizationCalibration
-from dkist_processing_visp.tasks.lamp import LampCalibration
-from dkist_processing_visp.tasks.make_movie_frames import MakeVispMovieFrames
-from dkist_processing_visp.tasks.parse import ParseL0VispInputData
-from dkist_processing_visp.tasks.science import ScienceCalibration
-from dkist_processing_visp.tasks.solar import SolarCalibration
-from dkist_processing_visp.tasks.split import SplitBeams
-from dkist_processing_visp.tasks.write_l1 import VispWriteL1Frame
-from dkist_processing_visp.tests.e2e_helpers import LoadDarkCal
-from dkist_processing_visp.tests.e2e_helpers import LoadGeometricCal
-from dkist_processing_visp.tests.e2e_helpers import LoadInstPolCal
-from dkist_processing_visp.tests.e2e_helpers import LoadLampCal
-from dkist_processing_visp.tests.e2e_helpers import LoadSolarCal
-from dkist_processing_visp.tests.e2e_helpers import SaveDarkCal
-from dkist_processing_visp.tests.e2e_helpers import SaveGeometricCal
-from dkist_processing_visp.tests.e2e_helpers import SaveInstPolCal
-from dkist_processing_visp.tests.e2e_helpers import SaveLampCal
-from dkist_processing_visp.tests.e2e_helpers import SaveSolarCal
-from dkist_processing_visp.visp_base import VispScienceTask
-
-
-class Translate122To214L0(WorkflowDataTaskBase):
+class Translate122To214L0(WorkflowTaskBase):
     def run(self) -> None:
         raw_dir = Path(self.scratch.scratch_base_path) / f"VISP{self.recipe_run_id:03n}"
         if not os.path.exists(self.scratch.workflow_base_path):
@@ -89,14 +86,18 @@ class Translate122To214L0(WorkflowDataTaskBase):
             header = spec122_validator.validate_and_translate_to_214_l0(
                 hdl[0].header, return_type=fits.HDUList
             )[0].header
-            hdl[0].header = header
 
-            hdl.writeto(translated_file_name, overwrite=True)
+            comp_hdu = fits.CompImageHDU(header=header, data=hdl[0].data)
+            comp_hdl = fits.HDUList([fits.PrimaryHDU(), comp_hdu])
+            comp_hdl.writeto(translated_file_name, overwrite=True)
+
             hdl.close()
             del hdl
+            comp_hdl.close()
+            del comp_hdl
 
 
-class CreateInputDatasetParameterDocument(WorkflowDataTaskBase):
+class CreateInputDatasetParameterDocument(WorkflowTaskBase):
     def run(self) -> None:
         doc_path = self.scratch.workflow_base_path / "dataset_doc.json"
         with open(doc_path, "w") as f:
@@ -108,7 +109,7 @@ class CreateInputDatasetParameterDocument(WorkflowDataTaskBase):
     def input_dataset_document_with_simple_parameters(self):
         dataset_doc_dict = dict(parameters=[])
         value_id = randint(1000, 2000)
-        for pn, pv in asdict(VispParameters()).items():
+        for pn, pv in asdict(VispTestingParameters()).items():
             values = [
                 {
                     "parameterValueId": value_id,
@@ -122,44 +123,80 @@ class CreateInputDatasetParameterDocument(WorkflowDataTaskBase):
         return dataset_doc_dict
 
 
-class TagInputs(WorkflowDataTaskBase):
+def tag_inputs_task(suffix: str):
+    class TagInputs(WorkflowTaskBase):
+        def run(self) -> None:
+            logging.info(f"Looking in {os.path.abspath(self.scratch.workflow_base_path)}")
+            input_file_list = list(self.scratch.workflow_base_path.glob(f"*.{suffix}"))
+            if len(input_file_list) == 0:
+                raise FileNotFoundError(
+                    f"Did not find any files matching '*.{suffix}' in {self.scratch.workflow_base_path}"
+                )
+            for file in input_file_list:
+                logging.info(f"Found {file}")
+                self.tag(path=file, tags=[VispTag.input(), VispTag.frame()])
+
+    return TagInputs
+
+
+class ShowPolMode(VispTaskBase):
     def run(self) -> None:
-        logging.info(f"Looking in {os.path.abspath(self.scratch.workflow_base_path)}")
-        for file in self.scratch.workflow_base_path.glob("*.FITS"):
-            logging.info(f"Found {file}")
-            self.tag(path=file, tags=[VispTag.input(), VispTag.frame()])
+        logging.info(f"{self.constants.correct_for_polarization = }")
 
 
-class ShowPolMode(VispScienceTask):
+class ShowExposureTimes(VispTaskBase):
     def run(self) -> None:
-        logging.info(f"{self.correct_for_polarization=}")
+        logging.info(f"{self.constants.dark_exposure_times = }")
+        logging.info(f"{self.constants.lamp_exposure_times = }")
+        logging.info(f"{self.constants.solar_exposure_times = }")
+        if self.constants.correct_for_polarization:
+            logging.info(f"{self.constants.polcal_exposure_times = }")
+        logging.info(f"{self.constants.observe_exposure_times = }")
 
 
-class ShowExposureTimes(VispScienceTask):
+class ShowMapMapping(VispTaskBase):
     def run(self) -> None:
-        logging.info(f"{self.dark_exposure_times = }")
-        logging.info(f"{self.lamp_exposure_times = }")
-        logging.info(f"{self.solar_exposure_times = }")
-        if self.correct_for_polarization:
-            logging.info(f"{self.polcal_exposure_times = }")
-        logging.info(f"{self.observe_exposure_times = }")
+        logging.info(f"Found {self.constants.num_map_scans} map scans")
+        step = 0
+        logging.info(f"Step {step} is organized thusly:")
+        for map in range(1, self.constants.num_map_scans + 1):
+            logging.info(f"Map #{map}:")
+            for mod in range(1, self.constants.num_modstates + 1):
+                fits_obj = list(
+                    self.fits_data_read_fits_access(
+                        tags=[
+                            VispTag.input(),
+                            VispTag.frame(),
+                            VispTag.map_scan(map),
+                            VispTag.modstate(mod),
+                            VispTag.raster_step(step),
+                        ],
+                        cls=VispL0FitsAccess,
+                    )
+                )
+                logging.info(
+                    f"\tModstate {mod} has {len(fits_obj)} files. Date is {fits_obj[0].time_obs}"
+                )
 
 
-class SubmitAndExposeQuality(ParsedL0InputTaskBase, QualityMixin, MetadataStoreMixin):
-    """ A direct copy paste of SumbitQuality with an additional step of writing the report to disk"""
+class SubmitAndExposeQuality(WorkflowTaskBase, QualityMixin, MetadataStoreMixin):
+    """A direct copy paste of SumbitQuality with an additional step of writing the report to disk"""
 
     def run(self):
-        logging.info("Building quality report")
-        report_str = self.quality_build_report()
-        logging.info("Submitting quality report")
-        self.metadata_store_add_quality_report(
-            dataset_id=self.dataset_id, quality_report=report_str
-        )
+        with self.apm_processing_step("Building quality report"):
+            logging.info("Building quality report")
+            report_str = self.quality_build_report()
+
+        with self.apm_task_step("Submitting quality report"):
+            logging.info("Submitting quality report")
+            self.metadata_store_add_quality_report(
+                dataset_id=self.constants.dataset_id, quality_report=report_str
+            )
 
         if QRM:
             doc_path = self.scratch.workflow_base_path / "quality_report.json"
             report_container = {
-                "datasetId": self.dataset_id,
+                "datasetId": self.constants.dataset_id,
                 "qualityReport": json.dumps(report_str, cls=DatetimeEncoder),
             }
             json_str = json.dumps(report_container)
@@ -168,7 +205,7 @@ class SubmitAndExposeQuality(ParsedL0InputTaskBase, QualityMixin, MetadataStoreM
             logging.info(f"Wrote report to {doc_path}")
 
 
-class ValidateL1Output(VispScienceTask):
+class ValidateL1Output(VispTaskBase):
     def run(self) -> None:
         files = self.read(tags=[VispTag.output(), VispTag.frame()])
         for f in files:
@@ -176,6 +213,21 @@ class ValidateL1Output(VispScienceTask):
             spec214_validator.validate(f, extra=False)
 
 
+def setup_APM_config() -> None:
+    mesh_config = {
+        "system-monitoring-log-apm": {
+            "mesh_address": "system-monitoring-log-apm.service.sim.consul",
+            "mesh_port": 8200,
+        },
+        "automated-processing-scratch-inventory": {"mesh_address": "localhost", "mesh_port": 6379},
+        "internal-api-gateway": {"mesh_address": "localhost", "mesh_port": 80},
+    }
+    apm_options = {"TRANSACTION_MAX_SPANS": 10000}
+    os.environ["MESH_CONFIG"] = json.dumps(mesh_config)
+    os.environ["ELASTIC_APM_ENABLED"] = "true"
+    os.environ["ELASTIC_APM_OTHER_OPTIONS"] = json.dumps(apm_options)
+
+
 def make_pdf_report(scratch_path: str, recipe_run_id: int) -> None:
     if not QRM:
         logging.info(
@@ -202,6 +254,8 @@ def make_pdf_report(scratch_path: str, recipe_run_id: int) -> None:
 def make_dataset_asdf(recipe_run_id, scratch_path):
     if not INV:
         logging.warning("Did NOT make dataset asdf file because dkist_inventory is not installed")
+        return
+
     output_dir = os.path.join(scratch_path, str(recipe_run_id))
     asdf_name = f"dataset_{recipe_run_id:03n}.asdf"
     logging.info(f"Creating ASDF file from {output_dir} and saving to {asdf_name}")
@@ -210,6 +264,7 @@ def make_dataset_asdf(recipe_run_id, scratch_path):
 
 def main(
     scratch_path: str,
+    suffix: str = "FITS",
     recipe_run_id: int = 2,
     skip_translation: bool = False,
     only_translate: bool = False,
@@ -218,21 +273,28 @@ def main(
     load_geometric: bool = False,
     load_solar: bool = False,
     load_inst_pol: bool = False,
+    use_apm: bool = False,
 ):
+    if use_apm:
+        setup_APM_config()
     with ManualProcessing(
-        workflow_path=scratch_path, recipe_run_id=recipe_run_id, testing=True
+        workflow_path=scratch_path,
+        recipe_run_id=recipe_run_id,
+        testing=True,
+        workflow_name="visp-l0-pipeline",
+        workflow_version="GROGU",
     ) as manual_processing_run:
         if not skip_translation:
             manual_processing_run.run_task(task=Translate122To214L0)
         if only_translate:
             return
         manual_processing_run.run_task(task=CreateInputDatasetParameterDocument)
-        manual_processing_run.run_task(task=TagInputs)
+        manual_processing_run.run_task(task=tag_inputs_task(suffix))
         manual_processing_run.run_task(task=ParseL0VispInputData)
-        manual_processing_run.run_task(task=QualityL0Metrics)
+        manual_processing_run.run_task(task=ShowMapMapping)
+        manual_processing_run.run_task(task=VispL0QualityMetrics)
         manual_processing_run.run_task(task=ShowPolMode)
         manual_processing_run.run_task(task=ShowExposureTimes)
-        manual_processing_run.run_task(task=SplitBeams)
         if load_dark:
             manual_processing_run.run_task(task=LoadDarkCal)
         else:
@@ -266,15 +328,20 @@ def main(
         manual_processing_run.run_task(task=ScienceCalibration)
         manual_processing_run.run_task(task=VispWriteL1Frame)
         manual_processing_run.run_task(task=QualityL1Metrics)
-        manual_processing_run.run_task(task=VispQualityMetrics)
+        manual_processing_run.run_task(task=VispL1QualityMetrics)
         manual_processing_run.run_task(task=SubmitAndExposeQuality)
         manual_processing_run.run_task(task=ValidateL1Output)
         manual_processing_run.run_task(task=MakeVispMovieFrames)
         manual_processing_run.run_task(task=AssembleVispMovie)
 
-    # Test some downstream services
-    make_dataset_asdf(recipe_run_id, scratch_path)
-    make_pdf_report(scratch_path, recipe_run_id)
+        # Test some downstream services
+        make_dataset_asdf(recipe_run_id, scratch_path)
+        make_pdf_report(scratch_path, recipe_run_id)
+
+        if any([load_dark, load_lamp, load_geometric, load_solar, load_inst_pol]):
+            logging.info("NOT counting provenance records because some tasks were skipped")
+        else:
+            manual_processing_run.count_provenance()
 
 
 if __name__ == "__main__":
@@ -289,6 +356,7 @@ if __name__ == "__main__":
         type=int,
         default=4,
     )
+    parser.add_argument("--suffix", help="File suffix to treat as INPUT frames", default="FITS")
     parser.add_argument(
         "-T",
         "--skip-translation",
@@ -328,10 +396,12 @@ if __name__ == "__main__":
         help="Load instrument polarization calibration from previously saved run",
         action="store_true",
     )
+    parser.add_argument("-A", "--use-apm", help="Send APM spans to SIM", action="store_true")
     args = parser.parse_args()
     sys.exit(
         main(
             scratch_path=args.scratch_path,
+            suffix=args.suffix,
             recipe_run_id=args.run_id,
             skip_translation=args.skip_translation,
             only_translate=args.only_translate,
@@ -340,5 +410,6 @@ if __name__ == "__main__":
             load_geometric=args.load_geometric,
             load_solar=args.load_solar,
             load_inst_pol=args.load_inst_pol,
+            use_apm=args.use_apm,
         )
     )
